function [ FFT_parameter, EigDecompDoubCurl_cell ] = EigDecompDoubCurl_SC( n1, n2, n3, vec_k, ...
    period_vec1, period_vec2, period_vec3, delta)
%UNTITLED Summary of this function goes here
%   Detailed explanation goes here

[mtx_D_xyz, Lambda_x, Lambda_y, Lambda_z] = construct_eigenvalue_mtx(n1, n2, ...
                   n3, vec_k, period_vec1, period_vec2, period_vec3, delta);
         
if ( norm(vec_k) == 0 )
    [Lambda_q, Lambda_Q1, Lambda_Q2] = construct_Lambda_dat_zero(Lambda_x, Lambda_y, Lambda_z);
    inv_Lambda_q_s                   = Lambda_q.^(-0.5);
%     [val,idx] = sort(Lambda_q);
%     Lambda_q(idx(1:10)).^(0.5)
else
    [inv_Lambda_q_s, Lambda_Q1, Lambda_Q2] = construct_Lambda_dat(Lambda_x, Lambda_y, Lambda_z);
end
% [inv_Lambda_q_s, Lambda_Q1, Lambda_Q2] = construct_Lambda_dat(Lambda_x, Lambda_y, Lambda_z);
            
FFT_parameter.n1         = n1;
FFT_parameter.n2         = n2;
FFT_parameter.n3         = n3;
FFT_parameter.mtx_D_xyz  = mtx_D_xyz; 

n                         = n1 * n2 * n3;
%
[i_idx_1, j_idx_1, val_1] = find(Lambda_Q1);
[i_idx_2, j_idx_2, val_2] = find(Lambda_Q2);
i_idx                     = [i_idx_1; i_idx_2];
j_idx                     = [j_idx_1; n+j_idx_2];
val                       = [val_1; val_2];

EigDecompDoubCurl_cell.Lambda_Qr      = sparse(i_idx, j_idx, val);
EigDecompDoubCurl_cell.inv_Lambda_q_s = inv_Lambda_q_s;
EigDecompDoubCurl_cell.Lambda         = conj(Lambda_x).*Lambda_x + ...
    conj(Lambda_y).*Lambda_y + conj(Lambda_z).*Lambda_z;
EigDecompDoubCurl_cell.Lambda_x       = Lambda_x;
EigDecompDoubCurl_cell.Lambda_y       = Lambda_y;
EigDecompDoubCurl_cell.Lambda_z       = Lambda_z;

end

% =========================================================================
function [mtx_D_xyz, Lambda_x, Lambda_y, Lambda_z] = construct_eigenvalue_mtx(n_1, n_2, ...
                    n_3, vec_k, period_vec1, period_vec2, period_vec3, delta)

    theta_k = 2 * pi * 1i * ( vec_k' * period_vec1 ) / n_1;
    mtx_D_x = exp(theta_k*(0:n_1-1)');

    theta_k = 2 * pi * 1i * ( vec_k' * period_vec2 ) / n_2;
    mtx_D_y = exp(theta_k*(0:n_2-1)');

    theta_k = 2 * pi * 1i * ( vec_k' * period_vec3 ) / n_3;
    mtx_D_z = exp(theta_k*(0:n_3-1)');
    
    mtx_D_zy  = kron(mtx_D_z, mtx_D_y);
    mtx_D_xyz = kron(mtx_D_zy, mtx_D_x);
      
    lambda_x = zeros(n_1,1);
    lambda_y = zeros(n_2,1);
    lambda_z = zeros(n_3,1);
        
    for j=1:n_1
        theta_j       = 2 * pi * 1i * ( (j-1) + vec_k' * period_vec1 ) / n_1;
        lambda_x(j,1) = ( exp(theta_j) - 1 ) / delta(1);
    end
    Lambda_x = kron(ones(n_2*n_3,1), lambda_x);
    

    for n=1:n_2
        phi_n         = 2 * pi * 1i * ( (n-1) + vec_k' * period_vec2 ) / n_2;
        lambda_y(n,1) = (exp(phi_n) - 1) / delta(2);
    end
    Lambda_y = kron(ones(n_3,1), lambda_y);
    Lambda_y = kron(Lambda_y, ones(n_1,1));
    
    for ii=1:n_3
        psi_l          = 2 * pi * 1i * ( (ii-1)+ vec_k' * period_vec3) / n_3;
        lambda_z(ii,1) = (exp(psi_l) - 1) / delta(3);
    end
    Lambda_z = kron(lambda_z, ones(n_1*n_2,1));
end

% =========================================================================
function [inv_Lambda_q_s, Lambda_Q1, Lambda_Q2] = construct_Lambda_dat(Lambda_x, Lambda_y, Lambda_z)

%global alpha beta

alpha          = 2;
beta           = 3;
Lambda_y2x     = Lambda_y - alpha * Lambda_x;
Lambda_2z3y    = alpha * Lambda_z - beta * Lambda_y;
Lambda_3xz     = beta * Lambda_x - Lambda_z;
Lambda_q       = conj(Lambda_x).*Lambda_x+conj(Lambda_y).*Lambda_y+conj(Lambda_z).*Lambda_z;
Lambda_p       = conj(Lambda_2z3y).*Lambda_2z3y + conj(Lambda_3xz).*Lambda_3xz + conj(Lambda_y2x).*Lambda_y2x; 
inv_Lambda_p_s = (real(Lambda_p)).^(-0.5);
        
%
% Compute inv_Lambda_q_s := (Lambda_x' * Lambda_x + Lambda_y' * Lambda_y + Lambda_z' * Lambda_z)^{-1/2}
inv_Lambda_q_s  = (real(Lambda_q)).^(-0.5);
Lambda_pq_inv_s = inv_Lambda_p_s.*inv_Lambda_q_s;
% %
% % Compute Lambda_q_s := (Lambda_x' * Lambda_x + Lambda_y' * Lambda_y + Lambda_z' * Lambda_z)^{1/2}
% Lambda_q_s      = (real(Lambda_q))^(0.5);

n       = length(Lambda_z);
idx     = zeros(3*n,1);
jdx     = zeros(3*n,1);
val     = zeros(3*n,1);
%Lambda_Q1 = zeros(3*n,1);
%Lambda_Q2 = zeros(3*n,1);

idx(    1:n,  1) = 1:n;
jdx(    1:n,  1) = 1:n;
val(    1:n,  1) = Lambda_y2x.*conj(Lambda_y) - Lambda_3xz.*conj(Lambda_z);
val(    1:n,  1) = val(    1:n,  1).*Lambda_pq_inv_s;
idx(  n+1:2*n,1) = n+1:2*n;
jdx(  n+1:2*n,1) = 1:n;
val(  n+1:2*n,1) = Lambda_2z3y.*conj(Lambda_z) - Lambda_y2x.*conj(Lambda_x);
val(  n+1:2*n,1) = val(  n+1:2*n,1).*Lambda_pq_inv_s;
idx(2*n+1:3*n,1) = 2*n+1:3*n;
jdx(2*n+1:3*n,1) = 1:n;
val(2*n+1:3*n,1) = Lambda_3xz.*conj(Lambda_x) - Lambda_2z3y.*conj(Lambda_y);
val(2*n+1:3*n,1) = val(2*n+1:3*n,1).*Lambda_pq_inv_s;
Lambda_Q1        = sparse(idx, jdx, val);

idx(    1:n,  1) = 1:n;
jdx(    1:n,  1) = 1:n;
val(    1:n,  1) = conj(Lambda_2z3y).*inv_Lambda_p_s;
idx(  n+1:2*n,1) = n+1:2*n;
jdx(  n+1:2*n,1) = 1:n;
val(  n+1:2*n,1) = conj(Lambda_3xz).*inv_Lambda_p_s;
idx(2*n+1:3*n,1) = 2*n+1:3*n;
jdx(2*n+1:3*n,1) = 1:n;
val(2*n+1:3*n,1) = conj(Lambda_y2x).*inv_Lambda_p_s;
Lambda_Q2        = sparse(idx, jdx, val);
end

% =========================================================================
function [Lambda_q, Lambda_Q1, Lambda_Q2] = construct_Lambda_dat_zero(Lambda_x, Lambda_y, Lambda_z)

%global alpha beta

alpha          = 2;
beta           = 3;
Lambda_y2x     = Lambda_y - alpha * Lambda_x;
Lambda_2z3y    = alpha * Lambda_z - beta * Lambda_y;
Lambda_3xz     = beta * Lambda_x - Lambda_z;
Lambda_q       = conj(Lambda_x).*Lambda_x+conj(Lambda_y).*Lambda_y+conj(Lambda_z).*Lambda_z;
Lambda_p       = conj(Lambda_2z3y).*Lambda_2z3y + conj(Lambda_3xz).*Lambda_3xz + conj(Lambda_y2x).*Lambda_y2x;
Lambda_p       = Lambda_p(2:end,1);
inv_Lambda_p_s = (real(Lambda_p)).^(-0.5);
        
%
% Compute inv_Lambda_q_s := (Lambda_x' * Lambda_x + Lambda_y' * Lambda_y + Lambda_z' * Lambda_z)^{-1/2}
Lambda_q        = real(Lambda_q(2:end,1));
inv_Lambda_q_s  = (Lambda_q).^(-0.5);
Lambda_pq_inv_s = inv_Lambda_p_s.*inv_Lambda_q_s;
% %
% % Compute Lambda_q_s := (Lambda_x' * Lambda_x + Lambda_y' * Lambda_y + Lambda_z' * Lambda_z)^{1/2}
% Lambda_q_s      = (real(Lambda_q))^(0.5);

n       = length(Lambda_z);
idx     = zeros(3*n-3,1);
jdx     = zeros(3*n-3,1);
val     = zeros(3*n-3,1);
%Lambda_Q1 = zeros(3*n,1);
%Lambda_Q2 = zeros(3*n,1);

idx(    1:n-1,  1) = 2:n;
jdx(    1:n-1,  1) = 1:n-1;
tmp                = Lambda_y2x.*conj(Lambda_y) - Lambda_3xz.*conj(Lambda_z);
val(    1:n-1,  1) = tmp(    2:n,  1).*Lambda_pq_inv_s;
idx(    n:2*n-2,1) = n+2:2*n;
jdx(    n:2*n-2,1) = 1:n-1;
tmp                = Lambda_2z3y.*conj(Lambda_z) - Lambda_y2x.*conj(Lambda_x);
val(    n:2*n-2,1) = tmp( 2:n,1).*Lambda_pq_inv_s;
idx(2*n-1:3*n-3,1) = 2*n+2:3*n;
jdx(2*n-1:3*n-3,1) = 1:n-1;
tmp                = Lambda_3xz.*conj(Lambda_x) - Lambda_2z3y.*conj(Lambda_y);
val(2*n-1:3*n-3,1) = tmp(2:n,1).*Lambda_pq_inv_s;
Lambda_Q1          = sparse(idx, jdx, val);

idx(    1:n-1,  1) = 2:n;
jdx(    1:n-1,  1) = 1:n-1;
val(    1:n-1,  1) = conj(Lambda_2z3y(2:n,1)).*inv_Lambda_p_s;
idx(    n:2*n-2,1) = n+2:2*n;
jdx(    n:2*n-2,1) = 1:n-1;
val(    n:2*n-2,1) = conj(Lambda_3xz(2:n,1)).*inv_Lambda_p_s;
idx(2*n-1:3*n-3,1) = 2*n+2:3*n;
jdx(2*n-1:3*n-3,1) = 1:n-1;
val(2*n-1:3*n-3,1) = conj(Lambda_y2x(2:n,1)).*inv_Lambda_p_s;
Lambda_Q2          = sparse(idx, jdx, val);
end

