function [EI,EB,ET,DPhi,VOL,FB,E] = EdgeElement(T,V)

T = sort(T,2);

TNO  = size(T,1);
DPhi = zeros(TNO,3,4);
u    = V(T(:,2),:) - V(T(:,1),:);     % u = e12 = V2 - V1
v    = V(T(:,3),:) - V(T(:,1),:);     % v = e13 = V3 - V1
w    = V(T(:,4),:) - V(T(:,1),:);     % w = e14 = V4 - V1

% Compute gradient of the nodal bases Phi
DPhi(:,:,4) = cross(u,v);
DPhi(:,:,2) = cross(v,w);
DPhi(:,:,3) = cross(w,u);
VOL         = dot(w,DPhi(:,:,4),2);       % volume of parallelepiped
w           = V(T(:,4),:) - V(T(:,2),:);  % w = e24 = V4 - V2
v           = V(T(:,3),:) - V(T(:,2),:);  % v = e23 = V3 - V2
DPhi(:,:,1) = cross(w,v);
for k = 1:4
    for j = 1:3
        DPhi(:,j,k) = DPhi(:,j,k)./VOL;
    end
end

% Compute the volume of each tetrahedron
orind          = VOL < 0;  % oriented indices
if ~isempty(orind)
    VOL(orind) = -VOL(orind);
end
VOL = VOL/6;

% Construct the edge indices E and the edge-of-tetrahedron table ET
ij         = [1 1 1 2 2 3, 2 3 4 3 4 4];
[E,~,temp] = unique(reshape(T(:,ij),6*TNO,2),'rows');
ET         = sort(reshape(temp,TNO,6),2);

% Detect the faces on the boundary
ij         = [1 1 1 2, 2 2 3 3, 3 4 4 4];
[F,~,temp] = unique(reshape(T(:,ij),4*TNO,3),'rows');
FB         = F(accumarray(temp, 1) == 1,:);

% Determine the boundary/interior edge indices EB and EI
isEB = sum(ismember(E,unique(FB)),2) == 2;
EB   = find( isEB);
EI   = find(~isEB);
%
end