function [M1,F1,G1,Mn,Fn,Gn,MM,MMn] = GetMassMat(EI,EB,ET,DPhi,VOL,NX)

% gradPhi : TNO x 3 x 4
% dGradPhi: 
% NX: 3 x 3 symmetric matrix with positive entries.

ENO   = max(max(ET));
TNO   = size(VOL,1);

dotDPhi = zeros(TNO,4,4);
for i = 1:4
    for j = i:4
        dotDPhi(:,i,j) = dot(DPhi(:,:,i),DPhi(:,:,j),2);
        dotDPhi(:,j,i) = dotDPhi(:,i,j);
    end
end

Bij = zeros(TNO,15);
eij = [1 2; 1 3; 1 4; 2 3; 2 4; 3 4];
k   = 1;
for i = 1:5
    for j = (i+1):6
        i1 = eij(i,1); i2 = eij(i,2);
        j1 = eij(j,1); j2 = eij(j,2);
        Bij(:,k) = 1/20*VOL.*(   dotDPhi(:,i2,j2)*(1+(i1==j1)) ...
                               - dotDPhi(:,i2,j1)*(1+(i1==j2)) ...
                               - dotDPhi(:,i1,j2)*(1+(i2==j1)) ...
                               + dotDPhi(:,i1,j1)*(1+(i2==j2))     );
        k = k + 1;
    end
end
ei = [1 1 1 1 1 2 2 2 2 3 3 3 4 4 5];
ej = [2 3 4 5 6 3 4 5 6 4 5 6 5 6 6];
MM  = sparse(ET(:,ei),ET(:,ej),Bij,ENO,ENO);

Bij = zeros(TNO,6);
for k = 1:6
    k1 = eij(k,1); k2 = eij(k,2);
    Bij(:,k) = (0.1*VOL).*(dotDPhi(:,k2,k2) - dotDPhi(:,k1,k2) + dotDPhi(:,k1,k1));
end                 
MM  = MM + MM' + sparse(ET(:,1:6),ET(:,1:6),Bij,ENO,ENO);

M1 = MM(EI,EI);
F1 = MM(EI,EB);
G1 = MM(EB,EB);

% non-constant case
if nargin == 6
    dotDPhi = zeros(TNO,4,4);
    for i = 1:4
        for j = i:4
            dotDPhi(:,i,j) = dot(DPhi(:,:,i)*NX,DPhi(:,:,j),2);
            dotDPhi(:,j,i) = dotDPhi(:,i,j);
        end
    end

    Bij = zeros(TNO,15);
    eij = [1 2; 1 3; 1 4; 2 3; 2 4; 3 4];
    k   = 1;
    for i = 1:5
        for j = (i+1):6
            i1 = eij(i,1); i2 = eij(i,2);
            j1 = eij(j,1); j2 = eij(j,2);
            Bij(:,k) = 1/20*VOL.*(   dotDPhi(:,i2,j2)*(1+(i1==j1)) ...
                                   - dotDPhi(:,i2,j1)*(1+(i1==j2)) ...
                                   - dotDPhi(:,i1,j2)*(1+(i2==j1)) ...
                                   + dotDPhi(:,i1,j1)*(1+(i2==j2))     );
            k = k + 1;
        end
    end
    ei = [1 1 1 1 1 2 2 2 2 3 3 3 4 4 5];
    ej = [2 3 4 5 6 3 4 5 6 4 5 6 5 6 6];
    MMn  = sparse(ET(:,ei),ET(:,ej),Bij,ENO,ENO);

    Bij = zeros(TNO,6);
    for k = 1:6
        k1 = eij(k,1); k2 = eij(k,2);
        Bij(:,k) = (0.1*VOL).*(dotDPhi(:,k2,k2) - dotDPhi(:,k1,k2) + dotDPhi(:,k1,k1));
    end                 
    MMn  = MMn + MMn' + sparse(ET(:,1:6),ET(:,1:6),Bij,ENO,ENO);
    
    Mn = MMn(EI,EI);
    Fn = MMn(EI,EB);
    Gn = MMn(EB,EB);
end





end
