% Load/Generate mesh data to get T (tetrahedrons) and V (nodes)
clear
format long e
%warning('OFF');


fid         = fopen('parameter.txt','r');
no_mesh    = fscanf(fid,'%d',1); 
shift       = fscanf(fid,'%g',1);
mu          = fscanf(fid,'%g',1); 
ref_idx     = fscanf(fid,'%g',1);
status      = fclose(fid);

s = RandStream('mt19937ar','Seed',0);
RandStream.setGlobalStream(s);
x = rand(3,1);

%NX = [ ref_idx   1   0 ;   1   ref_idx   0 ;   0    0   ref_idx-2 ];
NX = [ ref_idx   1   0.5 ;   1   ref_idx-1   0.8 ;   0.5    0.8   ref_idx-1.5 ];

    n_mesh_x = no_mesh;
    n_mesh_y = no_mesh;
    n_mesh_z = no_mesh;
    [T,V]    = boxTetraMesh(n_mesh_x,n_mesh_y,n_mesh_z);

% Generate the coefficient matrices
% B0, Y0, Mh are only used for the QEP to construct the matrix A0 
% [mtx_K,mtx_E,mtx_M,mtx_F,mtx_G] = GetQEPMat(T,V,NX);

[par.mtx_K,par.mtx_E,par.mtx_M1,par.mtx_F1,G1,Mn,Fn,Gn] = GetQEPMat(T,V,NX);

par.mtx_F      = Fn - par.mtx_F1;
par.mtx_M      = Mn - par.mtx_M1;
par.mtx_G      = Gn - G1;

clear T V

dim   = size(par.mtx_K,1);

fprintf('dimension_K = %6.0f; dim_G = %6.0f; \n', dim, size(par.mtx_G,1));
for ii = 1:3
    fprintf('NX(%1.0f,:) = [ %3.1f  %3.1f  %3.1f ]; \n', ii , NX(ii,1), NX(ii,2), NX(ii,3));
end

par.mu = mu;
mtx_A  = Coefficient_mtx( par, shift );

fprintf('dimension of M = %6.0f \n', size(mtx_A,1));

rhs   = [zeros(2*dim+2*size(par.mtx_G,1),1); randn(dim,1)];
tol   = 1.0e-10;
maxit = 10^5;
iter_method = 'BiCG';

switch iter_method
    case 'BiCGSTAB' 
        [sol,flag,relres,iter] = bicgstab(mtx_A, rhs, tol, maxit);  
        fprintf('%3.0f iterations of BiCGSTAB with flag = %2.0f and RelRes = %11.4e \n', ...
                 iter, flag, relres); 
            
    case 'BiCG'
        [sol,flag,relres,iter] = bicg(mtx_A, rhs, tol, maxit);  
        fprintf('%3.0f iterations of BiCG with flag = %2.0f and RelRes = %11.4e \n', ...
                 iter, flag, relres);
    case 'CGS' 
        [sol,flag,relres,iter] = cgs(mtx_A, rhs, tol, maxit);  
        fprintf('%3.0f iterations of CGS with flag = %2.0f and RelRes = %11.4e \n', ...
                 iter, flag, relres); 
            
    case 'GMRES'
        RESTRT                   = min(15,maxit); 
        [sol,flag,relres,no_iter] = gmres(mtx_A, rhs, RESTRT, tol, maxit);                     
        iter                  = no_iter(1) * RESTRT + no_iter(2);
        fprintf('%3.0f iterations of GMRES with flag = %2.0f and RelRes = %11.4e \n', ...
                 iter, flag, relres); 
              
    case 'BiCGSTABl' 
        [sol,flag,relres,iter] = bicgstabl(mtx_A, rhs, tol, maxit);  
        fprintf('%3.0f iterations of BiCGSTABl with flag = %2.0f and RelRes = %11.4e \n', ...
                 iter, flag, relres);
    case 'idrs'
        [sol,flag,relres,iter] = idrs(mtx_A, rhs, 4, tol, maxit);
        fprintf('%3.0f iterations of idrs with flag = %2.0f and RelRes = %11.4e \n', ...
                 iter, flag, relres);
             
    case 'lu'
        sol = mtx_A \ rhs;
             
end
