%
clear; %close all; clc; tic; run_time = toc;
%
format long e;
%delete rslt_dat2.txt
%diary rslt_dat2.txt
%diary on
%

warning('off','all')

fprintf('Choose test case \n');
fprintf('1. Solve shift linear system  (A - sigma B) x = b. \n'); 
fprintf('2. Solve FFT-based preconditioning linear system \n'); 
fprintf('3. Solve linear system K x = b in NFSEP\n'); 

test_case = input('==> ');

vec_k      = zeros(3,1);
fid        = fopen('runsh_parameter.txt','r');
n1        = fscanf(fid,'%d',1);
icase      = fscanf(fid,'%d',1);
icount     = fscanf(fid,'%d',1);
sigma      = fscanf(fid,'%g',1);
fclose(fid);


cubic_length   = 2 * pi; %2 * pi; %1; %2 * pi;

flag_eps = 1;
if ( flag_eps == 1 )
    ratio_epsilon  = 13; %1.3d1;   % Silicon
else
    ratio_epsilon   = 1.14d1;  % GaAs
end 

varepsilon_out      =  1.0;        % 0.3d0
varepsilon_in       = ratio_epsilon * varepsilon_out;

        period_vec1    = cubic_length * [ 1; 0; 0 ];
        period_vec2    = cubic_length * [ 0; 1; 0 ];
        period_vec3    = cubic_length * [ 0; 0; 1 ]; 

        length_x            = cubic_length;
        length_y            = cubic_length;
        length_z            = cubic_length;
    
        ratio_radius_length = 0.345;
        ratio_cyld_length   = 0.11;
        rd_sphere           = ratio_radius_length * cubic_length;
        rd_cylinder         = ratio_cyld_length * cubic_length; 
    
        n2                  = n1;  
        n3                  = n1;  
        delta               = [ length_x / n1; length_y / n2; length_z / n3]; 
        n                   = n1 * n2 * n3;
    
        diag_B_eps = ConstructMtxB_new(n1, n2, n3, delta, varepsilon_out, varepsilon_in, ...
            cubic_length, rd_sphere, rd_cylinder);
    
        no_wave_vec    = 11;
        tt             = linspace(0,1,no_wave_vec);

        vec_X          = [ 0.5; 0; 0];
        vec_M          = [ 0.5; 0.5; 0 ];
        vec_R          = [ 0.5; 0.5; 0.5];
        vec_G          = [ 0; 0; 0 ];

        switch icase
            case 1  
                vec_k  =  (1-tt(icount)) * vec_G + tt(icount) * vec_X; 
            
            case 2
                vec_k  =  (1-tt(icount)) * vec_X + tt(icount) * vec_M;
                
            case 3
                vec_k  =  (1-tt(icount)) * vec_M + tt(icount) * vec_R;
                
            case 4 
                vec_k  =  (1-tt(icount)) * vec_R + tt(icount) * vec_G;
        end

        vec_k = vec_k / cubic_length;

        %
        % =============== Construct EigDecompDoubCurl_cell ========================
        %            

        [ FFT_parameter, EigDecompDoubCurl_cell ] = EigDecompDoubCurl_SC( n1, n2, ...
                             n3, vec_k, period_vec1, period_vec2, period_vec3, delta );
                                
        % ==========================
        %

        mtx_C = Sparse_mtx_C_SC(n1,n2,n3,delta,period_vec1,period_vec2,period_vec3,vec_k);
        
        mtx_T_prod_vec  = @(input_vec)mtx_T_prod_vec_SC(input_vec, FFT_parameter);
        mtx_TH_prod_vec = @(input_vec)mtx_TH_prod_vec_SC(input_vec, FFT_parameter);
                  
s = RandStream('mt19937ar','Seed',0);
RandStream.setGlobalStream(s);

switch test_case
    case 1 % solve shift linear system (A - sigma B) x = b
        fprintf('Choose direct or iterative method \n')
        fprintf('1. Direct method \n');
        fprintf('2. Iterative method \n');
        solver = input('==> ');
        
        rhs   = randn(3*n,1) + 1i * randn(3*n,1);
        if ( solver == 1 )
            fprintf('\n');
            fprintf('matrix dimension is %10.0f \n',3*n);
            mtx_A = mtx_C' * mtx_C;
            mtx_B = sparse(1:3*n,1:3*n,diag_B_eps);
            tic;
            sol   = (mtx_A - sigma * mtx_B) \ rhs;
            fprintf('CPU time of direct method is %11.4e \n', toc);
            
        else
            fprintf('Choose preconditioner \n');
            fprintf('1. SSOR \n');
            fprintf('2. FFT-based \n');
            precond_case = input('==> ');
            
            maxit = 10^4;
            tol   = 1.0e-12;
            
            if ( precond_case == 1 )
                fprintf('\n');
                fprintf('matrix dimension is %10.0f \n',3*n);
            
                mtx_A          = mtx_C' * mtx_C;
                coef_mtx       = mtx_A - sigma * sparse(1:3*n,1:3*n,diag_B_eps);
                
                tic;
                upper_coef_mtx = triu(coef_mtx,1);
                diag_coef_mtx  = sparse(1:3*n,1:3*n,diag(coef_mtx));
                lower_coef_mtx = tril(coef_mtx,-1);
                SSOR_omega     = 0.8;
                lower_L        = diag_coef_mtx + SSOR_omega * lower_coef_mtx;
                upper_U        = diag_coef_mtx + SSOR_omega * upper_coef_mtx;
                
                %[lower_L, upper_U] = ilu(coef_mtx);
                
                [vec_t,flag,relres,iter] = bicgstabl(coef_mtx, rhs, tol, maxit, ...
                    @(x)SSOR_prec(x, diag_coef_mtx, lower_L, upper_U));  
                fprintf('Iteration number of bicgstabl is %3.0f with flag = %2.0f and RelRes = %11.4e; \n', ...
                  iter, flag, relres); 
                fprintf('CPU time is %11.4e \n', toc);
                
            else
                fprintf('Involve the matrix vector multiplication with A \n');
                fprintf('1. Yes  2. No\n');
                involve_A = input('==> ');
                
                tau      = sigma * sum(diag_B_eps) / (3*n);
                Lambda   = EigDecompDoubCurl_cell.Lambda - tau * ones(length(EigDecompDoubCurl_cell.Lambda),1);
                
                fprintf('\n');
                fprintf('Matrix dimension is %10.0f \n',3*n);
        
                if ( involve_A == 1 )
                    tic;
                    mtx_A    = mtx_C' * mtx_C;
                    coef_mtx = mtx_A - sigma * sparse(1:3*n,1:3*n,diag_B_eps);
                    [vec_t,flag,relres,iter] = bicgstabl(coef_mtx, rhs, tol, maxit, ...
                         @(vec)FFT_based_precond(vec, Lambda, tau, EigDecompDoubCurl_cell, ...
                         @(x)mtx_TH_prod_vec(x), @(x)mtx_T_prod_vec(x)));  
                    fprintf('Iteration number of bicgstabl is %3.0f with flag = %2.0f and RelRes = %11.4e; \n', ...
                         iter, flag, relres); 
                    fprintf('CPU time is %11.4e \n', toc);
                    
                    rsdl = norm(mtx_C' * (mtx_C * vec_t) - sigma * diag_B_eps.*vec_t - rhs);
                    fprintf('residual = %11.4e \n', rsdl);
                    
                else
                    LSopt.tol         = 1.0e-13;
                    LSopt.infoConvBeh = 'yes';
        
                    tic;
                    [ vec_t ] = solve_shift_invert_LS(sigma, rhs, diag_B_eps, EigDecompDoubCurl_cell, ...
                          @(x)mtx_TH_prod_vec(x), @(x)mtx_T_prod_vec(x), LSopt);
                    fprintf('CPU time is %11.4e \n', toc);
                    
                    rsdl = norm(mtx_C' * (mtx_C * vec_t) - sigma * diag_B_eps.*vec_t - rhs);
                    fprintf('residual = %11.4e \n', rsdl);

                end
            end
        end
        
    case 2 % solve preconditioning linear system
        rhs    = randn(3*n,1) + 1i * randn(3*n,1);
        
        fprintf('\n');
        fprintf('Matrix dimension is %10.0f \n',3*n);
        tic;
        tau    = sigma * sum(diag_B_eps) / (3*n);
        Lambda = EigDecompDoubCurl_cell.Lambda - tau * ones(length(EigDecompDoubCurl_cell.Lambda),1);
        
        y = FFT_based_precond(rhs, Lambda, tau, EigDecompDoubCurl_cell, ...
            @(x)mtx_TH_prod_vec(x), @(x)mtx_T_prod_vec(x));
        fprintf('CPU time for solving FFT-based preconditioning linear system is %11.4e \n',toc);
        
        rsdl = norm(mtx_C' * (mtx_C * y) - tau * y - rhs);
        fprintf('residual = %11.4e \n', rsdl);
        
    case 3 % Solve linear system K x = b in NFSEP
        
        LSopt.tol         = 1.0e-12;
        LSopt.infoConvBeh = 'yes';
        rhs          = randn(2*n,1) + 1i * randn(2*n,1);
        
        fprintf('Reformulate linear system or not \n');
        fprintf('1. Yes,  2. No\n');
        reform = input('==> ');
        
        fprintf('\n');
        fprintf('Matrix dimension is %10.0f \n',3*n);
        
        if ( reform == 2 )
            tic;
            
            maxit   = 1e4;
            tol     = 1.0e-12;

            [sol,flag,relres,iter] = pcg(@(vec)NFSEP_mtx_prod_vec_Lambda(vec, EigDecompDoubCurl_cell, ...
                  diag_B_eps, @(x)mtx_TH_prod_vec(x), @(x)mtx_T_prod_vec(x)), ...
                  rhs, tol, maxit);
    
            fprintf('Iteration number of CG is %4.0f with flag = %2.0f and relres = %11.4e; \n', ...
                  iter, flag, relres);

            fprintf('CPU time for solving linear system K x = b is %11.4e. \n',toc);
            
        else
            tic;
        
            sol = NFSEP_invert_LS( rhs, diag_B_eps, EigDecompDoubCurl_cell, ...
                  @(x)mtx_TH_prod_vec(x), @(x)mtx_T_prod_vec(x), LSopt );
        
            fprintf('CPU time for solving linear system K x = b is %11.4e. \n',toc);
        end
        
end
 
%
%diary off
