%
clear %close all; clc; tic; run_time = toc;
%
format long e;
%delete rslt_dat2.txt
%diary rslt_dat2.txt
%diary on
%
%warning('off','all')

vec_k      = zeros(3,1);
fid        = fopen('runsh_parameter.txt','r');
n1                  = fscanf(fid,'%d',1);
icase               = fscanf(fid,'%d',1);
icount              = fscanf(fid,'%d',1);
sigma               = fscanf(fid,'%g',1);
chirality_parameter = fscanf(fid,'%g',1);
fclose(fid);

cubic_length   = 1; %2 * pi; %2 * pi; %1; %2 * pi;

flag_eps = 1;
if ( flag_eps == 1 )
    ratio_epsilon  = 13; %1.3d1;   % Silicon
else
    ratio_epsilon   = 1.14d1;  % GaAs
end 

varepsilon_out      =  1.0;        % 0.3d0
varepsilon_in       = ratio_epsilon * varepsilon_out;

        period_vec1    = cubic_length * [ 1; 0; 0 ];
        period_vec2    = cubic_length * [ 0; 1; 0 ];
        period_vec3    = cubic_length * [ 0; 0; 1 ]; 

        length_x            = cubic_length;
        length_y            = cubic_length;
        length_z            = cubic_length;
    
        ratio_radius_length = 0.345;
        ratio_cyld_length   = 0.11;
        rd_sphere           = ratio_radius_length * cubic_length;
        rd_cylinder         = ratio_cyld_length * cubic_length; 
    
        n2                  = n1;  
        n3                  = n1;  
        delta               = [ length_x / n1; length_y / n2; length_z / n3]; 
        n                   = n1 * n2 * n3;
    
        diag_B_eps = ConstructMtxB_new(n1, n2, n3, delta, varepsilon_out, varepsilon_in, ...
            cubic_length, rd_sphere, rd_cylinder);
    
        no_wave_vec    = 11;
        tt             = linspace(0,1,no_wave_vec);

        vec_X          = [ 0.5; 0; 0];
        vec_M          = [ 0.5; 0.5; 0 ];
        vec_R          = [ 0.5; 0.5; 0.5];
        vec_G          = [ 0; 0; 0 ];

        switch icase
            case 1  
                vec_k  =  (1-tt(icount)) * vec_G + tt(icount) * vec_X; 
            
            case 2
                vec_k  =  (1-tt(icount)) * vec_X + tt(icount) * vec_M;
                
            case 3
                vec_k  =  (1-tt(icount)) * vec_M + tt(icount) * vec_R;
                
            case 4 
                vec_k  =  (1-tt(icount)) * vec_R + tt(icount) * vec_G;
        end

        vec_k = vec_k / cubic_length;

        %
        % =============== Construct EigDecompDoubCurl_cell ========================
        %            

        [ FFT_parameter, EigDecompDoubCurl_cell ] = EigDecompDoubCurl_SC( n1, n2, ...
                             n3, vec_k, period_vec1, period_vec2, period_vec3, delta );
                                
        % ==========================
        %

        mtx_C = Sparse_mtx_C_SC(n1,n2,n3,delta,period_vec1,period_vec2,period_vec3,vec_k);
        
        mtx_T_prod_vec  = @(input_vec)mtx_T_prod_vec_SC(input_vec, FFT_parameter);
        mtx_TH_prod_vec = @(input_vec)mtx_TH_prod_vec_SC(input_vec, FFT_parameter);
                  
        xi_d            = zeros(3*n,1);
        idx             = find( diag_B_eps == varepsilon_in );
        xi_d(idx,1)     = 1i * chirality_parameter * ones(length(idx),1);
        zeta_d          = - xi_d; 
        
        mtx_A = [ sigma * sparse(1:3*n,1:3*n,diag_B_eps)       mtx_C' + sigma * sparse(1:3*n,1:3*n,xi_d); ...
                  mtx_C - sigma * sparse(1:3*n,1:3*n,zeta_d)   -sigma * sparse(1:3*n,1:3*n,ones(3*n,1))];
            
s = RandStream('mt19937ar','Seed',0);
RandStream.setGlobalStream(s);

fprintf('Choose direct or iterative method \n')
        fprintf('1. Direct method \n');
        fprintf('2. Iterative method \n');
        solver = input('==> ');
        
        rhs   = randn(6*n,1) + 1i * randn(6*n,1);
        if ( solver == 1 )
            fprintf('\n');
            fprintf('matrix dimension is %10.0f \n',6*n); 
            tic;
            sol   = mtx_A \ rhs;
            fprintf('CPU time of direct method is %11.4e \n', toc);
            
        else 
            
            maxit = 10^3;
            tol   = 1.0e-12;
            
                fprintf('\n');
                fprintf('matrix dimension is %10.0f \n',6*n);
                            
                tic;
                upper_coef_mtx = triu(mtx_A,1);
                diag_coef_mtx  = sparse(1:6*n,1:6*n,diag(mtx_A));
                lower_coef_mtx = tril(mtx_A,-1);
                SSOR_omega     = 1;
                lower_L        = diag_coef_mtx + SSOR_omega * lower_coef_mtx;
                upper_U        = diag_coef_mtx + SSOR_omega * upper_coef_mtx;
                
                %[lower_L, upper_U] = ilu(coef_mtx);
                
                [vec_t,flag,relres,iter] = bicgstabl(mtx_A, rhs, tol, maxit, ...
                    @(x)SSOR_prec(x, diag_coef_mtx, lower_L, upper_U));  
                fprintf('Iteration number of bicgstabl is %3.0f with flag = %2.0f and RelRes = %11.4e; \n', ...
                  iter, flag, relres); 
                fprintf('CPU time is %11.4e \n', toc); 
        end
%
%diary off
