function [ vec_t ] = solve_shift_invert_LS(sigma, rhs, diag_B_eps, EigDecompDoubCurl_cell, ...
    fun_mtx_TH_prod_vec, fun_mtx_T_prod_vec, LSopt)
%Solve the linear system
%        ( A - tau I ) vec_t = rhs.
%    

iter_method  = 'BiCGSTABl'; %'CGS'; %'BiCGSTAB'; %'GMRES';
tol          = LSopt.tol; %max(1.0e-4, vec_r_tol^2); 
maxit        = 1e4;
%rhs_tmp      = rhs;

tau         = sigma * sum(diag_B_eps) / length(rhs);
Lambda_new  = EigDecompDoubCurl_cell.Lambda - tau * ones(length(EigDecompDoubCurl_cell.Lambda),1);
mtx_B_sigma = tau - sigma * diag_B_eps;

rhs        = FFT_based_precond(rhs, Lambda_new, tau, EigDecompDoubCurl_cell, ...
        fun_mtx_TH_prod_vec, fun_mtx_T_prod_vec); 

    switch iter_method
        case 'BiCGSTABl' 
            [vec_t,flag,relres,iter] = bicgstabl(@(vec)mtx_prod_vec_shift_invert_LS(vec, tau, Lambda_new, ...
                EigDecompDoubCurl_cell, mtx_B_sigma, fun_mtx_TH_prod_vec, ...
                fun_mtx_T_prod_vec), rhs, tol, maxit);  
            if ( strcmp(LSopt.infoConvBeh, 'YES') || strcmp(LSopt.infoConvBeh, 'yes') )
                fprintf('Iteration number of BiCGSTABl is %3.0f with flag = %2.0f and RelRes = %11.4e; \n', ...
                     iter, flag, relres); 
            end
 
        case 'GMRES'  
 
            RESTRT                   = min(15,maxit); % min(20,maxit);  % Restart number of GMRES
 
            [vec_t,flag,relres,iter] = gmres(@(vec)mtx_prod_vec_shift_invert_LS(vec, tau, Lambda_new, ...
                EigDecompDoubCurl_cell, mtx_B_sigma, fun_mtx_TH_prod_vec, ...
                fun_mtx_T_prod_vec), rhs, RESTRT, tol, maxit); 
            
            if ( strcmp(LSopt.infoConvBeh, 'YES') || strcmp(LSopt.infoConvBeh, 'yes') )
                no_iter                  = iter(1) * RESTRT + iter(2);
                fprintf('%3.0f iterations of GMRES with flag = %2.0f and relres = %11.4e\n', ...
                    no_iter,flag,relres); 
            end
         
        case 'CGS'
            [vec_t,flag,relres,iter] = cgs(@(vec)mtx_prod_vec_shift_invert_LS(vec, tau, Lambda_new, ...
                EigDecompDoubCurl_cell, mtx_B_sigma, fun_mtx_TH_prod_vec, ...
                fun_mtx_T_prod_vec), rhs, tol, maxit);  
            
            if ( strcmp(LSopt.infoConvBeh, 'YES') || strcmp(LSopt.infoConvBeh, 'yes') )
                fprintf('%3.0f iterations of CGS with flag = %2.0f and RelRes = %11.4e \n', iter, ...
                    flag, relres); 
            end
         
%         case 'BiCG'
%             [vec_t,flag,relres,iter] = bicg(@(vec)mtx_A_prod_vec_1231(vec, tau, Lambda_new, ...
%                 EigDecompDoubCurl_cell.Lambda_x, EigDecompDoubCurl_cell.Lambda_y, ...
%                 EigDecompDoubCurl_cell.Lambda_z, diag_B_sigma), rhs, tol, maxit);  
%             fprintf('%3.0f iterations of BiCG with flag = %2.0f and RelRes = %11.4e \n', iter, flag, relres)
%             total_iter_LS            = total_iter_LS + iter;
                 
    end

    
end

